/*
 * Copyright (c) 2012 Planet Telex Inc. all rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @fileOverview Contains the Slideshow plugin code.
 * @author <a href="mailto:rob@planettelex.net">Rob Dixon</a>
 */

/**
 * See (<a href="http://jquery.com">http://jquery.com</a>).
 * @name $
 * @class
 * <p>See the jQuery Library API (<a href="http://api.jquery.com/jQuery/">http://api.jquery.com/jQuery/</a>)
 * for full details.  This documentation only covers the functions and classes that are added to
 * jQuery by the Slideshow plugin.</p>
 */

/**
 * See (<a href="http://jquery.com">http://jquery.com</a>).
 * @name $.fn
 * @class
 * <p>The slideshow plugin interface. See jQuery (<a href="http://docs.jquery.com/Plugins/Authoring">http://docs.jquery.com/Plugins/Authoring</a>)
 * for the full details on authoring plugins.  This documentation only covers the functions and classes that are added to
 * jQuery by the Slideshow plugin.</p>
 *
 * @memberOf $
 */

(function($) {
    // Slideshow Variables
    var currentSlideIndex = 0,
		timeout = null,
		slides = [],
		thumbnails = [],
		activeThumbnails = [],
		carouselCursorStart = 0,
		carouselCursorEnd = 0,
		carouselButtonNext = null,
		carouselButtonPrevious = null,
		carouselCurrentScrollDirection = "Left",
		carouselThumbnails = 0,
		carouselWidth = "100px",
		carouselHeight = "50px",
		dotButtonActive = null,
		dotButtonInactive = null,
		nextButton = null,
		pauseButton = null,
		playButton = null,
		previousButton = null,
		slideshowId = "#Slideshow",
		slideshowDataUrl = "slideshow.xml";

    /**
    * <p>Creates the slideshow on the selected element. This plugin is intended to work with
    * an <a href="http://api.jquery.com/id-selector/">ID Selector</a> since it is generally best
    * practice to have only one slideshow on a page. Unexpected results will occur if multiple
    * elements are matched by the jQuery selector.</p>
    *
    * <b>Example XML configuration:</b>
    * <pre class="code">
    * 	&lt;?xml version="1.0" encoding="iso-8859-1"?&gt;
    * 	&lt;slideshow transitionMode="None" transitionDuration="2000"&gt;
    * 		&lt;slides secondsBetweenSlides="3"&gt;
    * 			&lt;slide src="demos/Cityscapes/city-at-sunset.jpg"
    * 			href="http://www.planettelex.net" target="_blank" /&gt;
    * 			&lt;slide src="demos/Cityscapes/docks.jpg" /&gt;
    * 			&lt;slide src="demos/Cityscapes/green-waters.jpg" /&gt;
    * 			&lt;slide src="demos/Cityscapes/quiet-city.jpg" /&gt;
    * 			&lt;slide src="demos/Cityscapes/skyline.jpg" /&gt;
    * 		&lt;/slides&gt;
    * 		&lt;dots&gt;
    * 			&lt;dot type="Active" src="images/dot-active.png" /&gt;
    * 			&lt;dot type="Inactive" src="images/dot-inactive.png" /&gt;
    * 		&lt;/dots&gt;
    * 	&lt;/slideshow&gt;
    * </pre>
    *
    * <b>Example usage:</b>
    * @example $("#Slideshow").slideshow("demos/Beaches/slideshow.xml");
    * @example jQuery("#Slideshow").slideshow("demos/Beaches/slideshow.xml");
    *
    * @param {String} dataUrl The url path to the slideshow XML configuration, can be a full or relative path.
    */
    $.fn.slideshow = function(dataUrl) {
        return (
			this.each(function() {
			    var el = $(this);
			    $.slideshow.startSlideshow("#" + el.attr("id"), dataUrl);
			})
		);
    };

    /**
    * <p>Static functions that are added to jQuery.slideshow namespace,
    * these functions may be used to control the slideshow. The startSlideshow method may be
    * used as an alternative to calling the slideshow plugin (see {@link $.fn.slideshow}).</p>
    *
    * <b>Example usage:</b>
    * @example $.slideshow.startSlideshow("#Slideshow", "demos/Beaches/slideshow.xml");
    * @example $.slideshow.setCarouselSlideDirection("Right");
    * @example $.slideshow.stopSlideshow();
    *
    * @namespace
    * <p>The slideshow static function namespace.</p>
    */
    $.slideshow = {
        /**
        * <p>Settings used to control the slideshow.</p>
        *
        * <b>Example usage:</b>
        * @example $.slideshow.settings.transitionMode = "Slide";
        * @example jQuery.slideshow.settings.transitionMode = "Slide";
        *
        * @class
        * Configurable properties of the slideshow and carousel.
        */
        settings: {
            /**Up|Down|Left|Right|Random|Infer, defaults to Infer*/carouselScrollDirection: "Infer",
            /**Time in milliseconds, defaults to 800*/carouselScrollDuration: 800,
            /**Time in seconds, defaults to 3*/secondsBetweenSlides: 3,
            /**Height in pixels, default inferred from CSS*/slideshowHeight: 0,
            /**Width in pixels, default inferred from CSS*/slideshowWidth: 0,
            /**None|Fade|Slide|Random, defaults to None*/transitionMode: "None",
            /**Up|Down|Left|Right|Random, defaults to Left*/transitionDirection: "Left",
            /**Time in milliseconds, defaults to 800*/transitionDuration: 800
        },

        /**
        * Starts the slideshow, if calling for an existing slideshow, that slideshow must first be stopped.
        *
        * @public
        * @static
        * @param {String} [id] The id of the element containing the slideshow, defaults to #Slideshow.
        * @param {String} [dataUrl] The url of the slideshow XML configuration file, defaults to slideshow.xml.
        */
        startSlideshow: function(id, dataUrl) {
            if (id != null)
                slideshowId = id;
            if (dataUrl != null)
                slideshowDataUrl = dataUrl;

            $.ajax({ type: "GET", url: slideshowDataUrl, success: loadSlideshowData, dataType: "xml" });
        },

        /**
        * Sets the carousel slide direction.
        *
        * @public
        * @static
        * @param {String} direction Up|Down|Left|Right|Random|Infer, defaults to {@link $.slideshow.settings.carouselScrollDirection}.
        */
        setCarouselSlideDirection: function(direction) {
            if (direction == null)
                direction = $.slideshow.settings.carouselScrollDirection;

            if (direction == "Random")
                carouselCurrentScrollDirection = getRandomDirection();
            else if (direction == "Infer") {
                if ($("#Slideshow #Carousel .carouselThumbnails img").css("float") == "left")
                    carouselCurrentScrollDirection = "Left";
                else if ($("#Slideshow #Carousel .carouselThumbnails li").css("float") == "left")
                    carouselCurrentScrollDirection = "Left";
                else if ($("#Slideshow #Carousel .carouselThumbnails img").css("float") == "right")
                    carouselCurrentScrollDirection = "Right";
                else if ($("#Slideshow #Carousel .carouselThumbnails li").css("float") == "right")
                    carouselCurrentScrollDirection = "Right";
                else
                    carouselCurrentScrollDirection = "Down";
            }
            else
                carouselCurrentScrollDirection = direction;

            var nextCoords = getNextThumbnailCoords();
            $(slideshowId + " #Carousel #NextThumbnails").css("top", nextCoords[0]);
            $(slideshowId + " #Carousel #NextThumbnails").css("left", nextCoords[1]);
        },

        /**
        * Shows the next slide using the transitionMode specified.
        *
        * @public
        * @static
        */
        showNextSlide: function() {
            currentSlideIndex = currentSlideIndex + 1 >= slides.length ? 0 : currentSlideIndex + 1;
            $.slideshow.scrollCarousel();

            var transition = $.slideshow.settings.transitionMode;
            if ($.slideshow.settings.transitionMode == "Random") {
                var randomNumber = Math.floor(Math.random() * 3);
                if (randomNumber == 1)
                    transition = "Fade";
                else if (randomNumber == 2)
                    transition = "Slide";
                else
                    transition = "None";
            }
            var slide = slides[currentSlideIndex];
            if (transition == "Fade")
                crossFadeSlides(slide);
            else if (transition == "Slide")
                slideSlides(slide);
            else
                swapSlides(slide);
        },

        /**
        * Scolls the carousel.
        *
        * @public
        * @static
        * @param {Bool} forceCarousel Forces the carousel to scroll even if the cursor is not at the end.
        */
        scrollCarousel: function(forceCarousel) {
            if (($(slideshowId + " #Carousel #NextThumbnails") != null) &&
		        (forceCarousel ||
		         (carouselCursorStart < carouselCursorEnd && (currentSlideIndex < carouselCursorStart || currentSlideIndex > carouselCursorEnd)) ||
		         (currentSlideIndex < carouselCursorStart && currentSlideIndex > carouselCursorEnd))) {
                carouselCursorStart = currentSlideIndex;
                carouselCursorEnd = carouselCursorStart + (carouselThumbnails - 1);
                if (carouselCursorEnd >= slides.length)
                    carouselCursorEnd = carouselCursorEnd - slides.length;

                loadNextThumbnails();
                $(slideshowId + " #Carousel #Thumbnails").animate(
		            slideDirectionCss(carouselHeight, carouselWidth, carouselCurrentScrollDirection),
		            $.slideshow.settings.carouselScrollDuration,
		            "swing");
                $(slideshowId + " #Carousel #NextThumbnails").animate(
		            slideDirectionCss(0, 0, carouselCurrentScrollDirection),
		            $.slideshow.settings.carouselScrollDuration,
		            "swing",
		            function() {
		                $(slideshowId + " #Carousel #Thumbnails").attr("id", "LastThumbnails");
		                $(slideshowId + " #Carousel #NextThumbnails").attr("id", "Thumbnails");

		                var nextCoords = getNextThumbnailCoords();
		                $(slideshowId + " #Carousel #LastThumbnails").css("top", nextCoords[0]);
		                $(slideshowId + " #Carousel #LastThumbnails").css("left", nextCoords[1]);
		                $(slideshowId + " #Carousel #LastThumbnails").attr("id", "NextThumbnails");

		                $.slideshow.setCarouselSlideDirection();
		            });
            }
        },

        /**
        * Toggles the slideshow between play and pause mode.
        *
        * @public
        * @static
        */
        playPauseToggle: function() {
            if ($(slideshowId + " #PlayPauseButton img").attr("id") == "PlayButton")
                $.slideshow.resumeSlideshow();
            else
                $.slideshow.pauseSlideshow();
        },

        /**
        * Pauses the slideshow.
        *
        * @public
        * @static
        */
        pauseSlideshow: function() {
            $.slideshow.goToSlide(currentSlideIndex, false);
            if (timeout != null)
                clearTimeout(timeout);
            $(slideshowId + " #PlayPauseButton").empty();
            $(slideshowId + " #PlayPauseButton").append(playButton);
        },

        /**
        * Resumes the paused slideshow.
        *
        * @public
        * @static
        */
        resumeSlideshow: function() {
            $.slideshow.resetPlayPauseButton();
            $.slideshow.showNextSlide();
        },

        /**
        * Sets the play/pause button back to its play mode which displays the pause button.
        *
        * @public
        * @static
        */
        resetPlayPauseButton: function() {
            if (playButton != null && pauseButton != null) {
                $(slideshowId + " #PlayPauseButton").empty();
                $(slideshowId + " #PlayPauseButton").append(pauseButton);
            }
        },

        /**
        * Changes slide to the next slide using no effects and stopping any running effects.
        */
        nextSlide: function() {
            var nextSlide = currentSlideIndex + 1 >= slides.length ? 0 : currentSlideIndex + 1;
            $.slideshow.goToSlide(nextSlide);
        },

        /**
        * Scrolls to the next carousel group and updates the slideshow accordingly.
        *
        * @public
        * @static
        */
        nextCarouselGroup: function() {
            var nextSlide = carouselCursorEnd + 1;
            if (nextSlide >= slides.length)
                nextSlide = 0;
            $.slideshow.goToSlide(nextSlide);
            $.slideshow.scrollCarousel();
        },

        /**
        * Changes slide to the previous slide using no effects and stopping any running effects.
        *
        * @public
        * @static
        */
        previousSlide: function() {
            var previousSlide = currentSlideIndex == 0 ? slides.length - 1 : currentSlideIndex - 1;
            $.slideshow.goToSlide(previousSlide);
        },

        /**
        * Scrolls to the previous carousel group and updates the slideshow accordingly.
        *
        * @public
        * @static
        */
        previousCarouselGroup: function() {
            var previousSlide = carouselCursorStart - carouselThumbnails;
            if (previousSlide < 0)
                previousSlide = slides.length + previousSlide;
            setCarouselDirection();
            $.slideshow.goToSlide(previousSlide, true);
        },

        /**
        * Changes slide to the specified slide using no effects and stopping any running effects.
        *
        * @public
        * @static
        * @param {Integer} slideIndex
        * @param {Boolean} forceCarouselScroll
        */
        goToSlide: function(slideIndex, forceCarouselScroll) {
            currentSlideIndex = slideIndex;
            $.slideshow.scrollCarousel(forceCarouselScroll);
            $(slideshowId + " #FrontImage").stop();
            clearTimeout(timeout);
            swapSlides(slides[slideIndex]);
            $(slideshowId + " #FrontImage").attr("style", "position: absolute; top: 0; left: 0; z-index: 100; border: 0; width: " + $.slideshow.settings.slideshowWidth + "; height: " + $.slideshow.settings.slideshowHeight + ";");
            $(slideshowId + " #FrontImage").show();
            $.slideshow.resetPlayPauseButton();
        },

        /**
        * Stops the slideshow and unloads its data. It can be started again by calling startSlideshow.
        *
        * @public
        * @static
        */
        stopSlideshow: function() {
            $(slideshowId + " #FrontImage").stop();
            clearTimeout(timeout);
            $(slideshowId).empty();

            $.slideshow.settings.slideshowWidth = 0;
            $.slideshow.settings.slideshowHeight = 0;
            $.slideshow.settings.carouselScrollDirection = "Left";
            $.slideshow.settings.carouselScrollDuration = 800;

            currentSlideIndex = 0;
            playButton = null;
            pauseButton = null;
            nextButton = null;
            previousButton = null;
            timeout = null;
            carouselButtonNext = null;
            carouselButtonPrevious = null;
            dotButtonActive = null;
            dotButtonInactive = null;
            carouselCursorStart = 0;
            carouselCursorEnd = 0;
            carouselCurrentScrollDirection = "Infer";
            carouselThumbnails = 0;
            slides = new Array();
            thumbnails = new Array();
            activeThumbnails = new Array();
        }
    };

    // Callback method for XML data ajax request, invoked from startSlideshow.
    function loadSlideshowData(xml) {
        if ($("slideshow", xml).attr("transitionMode") != null)
            $.slideshow.settings.transitionMode = $("slideshow", xml).attr("transitionMode");
        if ($("slideshow", xml).attr("transitionDuration") != null)
            transitionDuration = parseInt($("slideshow", xml).attr("transitionDuration"));
        if ($("slideshow", xml).attr("transitionDirection") != null)
            $.slideshow.settings.transitionDirection = $("slideshow", xml).attr("transitionDirection");
        if ($("slides", xml).attr("secondsBetweenSlides") != null)
            $.slideshow.settings.secondsBetweenSlides = parseInt($("slides", xml).attr("secondsBetweenSlides"));

        loadSlides(xml);
        loadButtons(xml);
        loadDots(xml);
        loadCarousel(xml);
        initializeMarkup();
        startTimer();
    }

    // Loads slides from the XML into the slides Array.
    function loadSlides(xml) {
        $("slide", xml).each(function() {
            var slide = new Object();
            slide.src = $(this).attr("src");
            slide.href = $(this).attr("href");
            slide.target = $(this).attr("target");
            slide.alt = $(this).attr("alt");
            slide.message = $("message", this).text();
            if ($("imageMap", this) != null) {
                var areas = new Array();
                $("area", this).each(function() {
                    var area = new Object();
                    area.alt = $(this).attr("alt");
                    area.href = $(this).attr("href");
                    area.target = $(this).attr("target");
                    area.shape = $(this).attr("shape");
                    area.coords = $(this).attr("coords");
                    areas.push(area);
                });
                slide.areas = areas;
            }
            var thumbnail = null;
            if ($("thumbnail", this) != null && $("thumbnail", this).attr("src") != null)
                thumbnail = $("thumbnail", this).attr("src");

            var activeThumbnail = null;
            if ($("thumbnail", this) != null && $("thumbnail", this).attr("activeSrc") != null)
                activeThumbnail = $("thumbnail", this).attr("activeSrc");

            activeThumbnails.push(activeThumbnail);
            thumbnails.push(thumbnail);
            slides.push(slide);
        });
    }

    // Loads the play, pause, next, and previous buttons.
    function loadButtons(xml) {
        if ($("buttons", xml) != null && $("buttons", xml).length > 0) {
            $("button", xml).each(function() {
                if ($(this).attr("type") != null) {
                    if ($(this).attr("type") == "Play") {
                        playButton = document.createElement("img");
                        playButton.setAttribute("id", "PlayButton");
                        playButton.setAttribute("src", $(this).attr("src"));
                    }
                    if ($(this).attr("type") == "Pause") {
                        pauseButton = document.createElement("img");
                        pauseButton.setAttribute("id", "PauseButton");
                        pauseButton.setAttribute("src", $(this).attr("src"));
                    }
                    if ($(this).attr("type") == "Next") {
                        nextButton = document.createElement("img");
                        nextButton.setAttribute("src", $(this).attr("src"));
                    }
                    if ($(this).attr("type") == "Previous") {
                        previousButton = document.createElement("img");
                        previousButton.setAttribute("src", $(this).attr("src"));
                    }
                }
            });
        }
    }

    // Loads the dot buttons.
    function loadDots(xml) {
        if ($("dots", xml) != null && $("dots", xml).length > 0) {
            $("dot", xml).each(function() {
                if ($(this).attr("type") == "Active") {
                    dotButtonActive = document.createElement("img");
                    dotButtonActive.setAttribute("class", "activeDotButton");
                    dotButtonActive.setAttribute("src", $(this).attr("src"));
                }
                if ($(this).attr("type") == "Inactive") {
                    dotButtonInactive = document.createElement("img");
                    dotButtonInactive.setAttribute("class", "dotButton");
                    dotButtonInactive.setAttribute("src", $(this).attr("src"));
                }
            });
        }
    }

    // Loads the carousel.
    function loadCarousel(xml) {
        if ($("carousel", xml) != null && $("carousel", xml).length > 0) {
            if ($("carousel", xml).attr("thumbnails") != null) {
                carouselThumbnails = parseInt($("carousel", xml).attr("thumbnails"));
                carouselCursorEnd = carouselThumbnails >= slides.length ? slides.length - 1 : carouselThumbnails - 1;
            }
            else
                carouselCursorEnd = slides.length - 1;

            if ($("carousel", xml).attr("scrollDirection") != null)
                $.slideshow.settings.carouselScrollDirection = $("carousel", xml).attr("scrollDirection");

            if ($("carousel", xml).attr("scrollDuration") != null)
                $.slideshow.settings.carouselScrollDuration = parseInt($("carousel", xml).attr("scrollDuration"));

            if ($("carouselButtons", xml) != null && $("carouselButtons", xml).length > 0) {
                $("carouselButton", xml).each(function() {
                    if ($(this).attr("type") == "Next") {
                        carouselButtonNext = document.createElement("a");
                        carouselButtonNext.setAttribute("id", "CarouselNextButton");
                        carouselButtonNext.setAttribute("class", "carouselButton");
                        carouselButtonNext.setAttribute("href", "JavaScript:jQuery.slideshow.nextCarouselGroup();");
                        carouselButtonNextImage = document.createElement("img");
                        carouselButtonNextImage.setAttribute("src", $(this).attr("src"));
                        carouselButtonNext.appendChild(carouselButtonNextImage);
                    }
                    if ($(this).attr("type") == "Previous") {
                        carouselButtonPrevious = document.createElement("a");
                        carouselButtonPrevious.setAttribute("id", "CarouselPreviousButton");
                        carouselButtonPrevious.setAttribute("class", "carouselButton");
                        carouselButtonPrevious.setAttribute("href", "JavaScript:jQuery.slideshow.previousCarouselGroup();");
                        carouselButtonPreviousImage = document.createElement("img");
                        carouselButtonPreviousImage.setAttribute("src", $(this).attr("src"));
                        carouselButtonPrevious.appendChild(carouselButtonPreviousImage);
                    }
                });
            }
        }
    }

    // Initializes the slideshow markup.
    function initializeMarkup() {
        var initialSlide = slides[0];

        var frontImage = document.createElement("img");
        frontImage.setAttribute("id", "FrontImage");
        frontImage.setAttribute("src", initialSlide.src);
        frontImage.setAttribute("usemap", "#ImageMap");
        frontImage.setAttribute("style", "position: absolute; top: 0; left: 0; z-index: 100; border: 0;");

        var backImage = document.createElement("img");
        backImage.setAttribute("id", "BackImage");
        backImage.setAttribute("src", initialSlide.src);
        backImage.setAttribute("style", "position: absolute; top: 0; left: 0; z-index: 0; border: 0;");

        var anchor = document.createElement("a");
        anchor.setAttribute("id", "SlideImage");
        anchor.setAttribute("style", "position: relative; overflow: hidden; display: block;");
        anchor.appendChild(frontImage);
        anchor.appendChild(backImage);

        var message = document.createElement("div");
        message.setAttribute("id", "Message");
        message.setAttribute("style", "display: none");

        var imageMap = document.createElement("map");
        imageMap.setAttribute("id", "ImageMap");
        imageMap.setAttribute("name", "ImageMap");

        $(slideshowId).append(anchor);
        $(slideshowId).append(message);
        $(slideshowId).append(imageMap);

        initializeButtons();
        initializeDots();
        initializeCarousel();

        if ($(slideshowId).css("width") != null) {
            $.slideshow.settings.slideshowWidth = $(slideshowId).css("width");
            $(slideshowId + " #SlideImage").css("width", $.slideshow.settings.slideshowWidth);
            $(slideshowId + " #FrontImage").css("width", $.slideshow.settings.slideshowWidth);
            $(slideshowId + " #BackImage").css("width", $.slideshow.settings.slideshowWidth);
        }

        if ($(slideshowId).css("height") != null) {
            $.slideshow.settings.slideshowHeight = $(slideshowId).css("height");
            $(slideshowId + " #SlideImage").css("height", $.slideshow.settings.slideshowHeight);
            $(slideshowId + " #FrontImage").css("height", $.slideshow.settings.slideshowHeight);
            $(slideshowId + " #BackImage").css("height", $.slideshow.settings.slideshowHeight);
        }

        updateMarkup(initialSlide);
    }

    // Starts the timer for the interval secondsBetweenSlides, at the end of which showNextSlide is called.
    function startTimer() {
        if (timeout != null)
            clearTimeout(timeout);
        timeout = setTimeout($.slideshow.showNextSlide, $.slideshow.settings.secondsBetweenSlides * 1000);
    }

    // Initializes the slideshow buttons.
    function initializeButtons() {
        if (playButton != null && pauseButton != null) {
            var playPauseButton = document.createElement("a");
            playPauseButton.setAttribute("id", "PlayPauseButton");
            playPauseButton.setAttribute("href", "JavaScript:jQuery.slideshow.playPauseToggle();");
            playPauseButton.appendChild(pauseButton);
            $(slideshowId).append(playPauseButton);
        }
        if (previousButton != null) {
            var previousSlideButton = document.createElement("a");
            previousSlideButton.setAttribute("id", "PreviousButton");
            previousSlideButton.setAttribute("href", "JavaScript:jQuery.slideshow.previousSlide();");
            previousSlideButton.appendChild(previousButton);
            $(slideshowId).append(previousSlideButton);
        }
        if (nextButton != null) {
            var nextSlideButton = document.createElement("a");
            nextSlideButton.setAttribute("id", "NextButton");
            nextSlideButton.setAttribute("href", "JavaScript:jQuery.slideshow.nextSlide();");
            nextSlideButton.appendChild(nextButton);
            $(slideshowId).append(nextSlideButton);
        }
    }

    // Initializes the slideshow dots.
    function initializeDots() {
        if (dotButtonActive != null && dotButtonInactive != null) {
            var dotsList = document.createElement("ul");
            dotsList.setAttribute("id", "Dots");
            dotsList.setAttribute("style", "list-style-type: none;");
            for (var i = 0; i < slides.length; i++) {
                var dotListItem = document.createElement("li");
                dotListItem.setAttribute("id", "Dot_" + i);
                var dotListLink = document.createElement("a");
                dotListLink.setAttribute("href", "JavaScript:jQuery.slideshow.goToSlide(" + i + ");");
                var dotImage = document.createElement("img");
                if (i == currentSlideIndex) {
                    dotImage.setAttribute("src", dotButtonActive.getAttribute("src"));
                    dotImage.setAttribute("class", dotButtonActive.getAttribute("class"));
                }
                else {
                    dotImage.setAttribute("src", dotButtonInactive.getAttribute("src"));
                    dotImage.setAttribute("class", dotButtonInactive.getAttribute("class"));
                }
                dotListLink.appendChild(dotImage);
                dotListItem.appendChild(dotListLink);
                dotsList.appendChild(dotListItem);
            }
            $(slideshowId).append(dotsList);
        }
    }

    // Initializes the slideshow carousel.
    function initializeCarousel() {
        if (carouselCursorEnd > 0) {
            var carousel = document.createElement("div");
            carousel.setAttribute("id", "Carousel");
            carousel.setAttribute("style", "overflow: hidden;");
            $(slideshowId).append(carousel);

            if ($(slideshowId + " #Carousel").css("width") != null)
                carouselWidth = $(slideshowId + " #Carousel").css("width");

            if ($(slideshowId + " #Carousel").css("height") != null)
                carouselHeight = $(slideshowId + " #Carousel").css("height");

            if ($(slideshowId + " #Carousel").css("position") == null || $(slideshowId + " #Carousel").css("position") == "static")
                $(slideshowId + " #Carousel").css("position", "relative");

            var thumbnailsList = document.createElement("ul");
            thumbnailsList.setAttribute("id", "Thumbnails");
            thumbnailsList.setAttribute("class", "carouselThumbnails");
            thumbnailsList.setAttribute("style", "position: absolute; width: " + carouselWidth + "; height: " + carouselHeight + "; list-style-type: none; top: 0; left: 0;");
            for (var i = 0; i <= carouselCursorEnd; i++)
                thumbnailsList.appendChild(buildCarouselListItem(i));

            var nextThumbnailsList = document.createElement("ul");
            nextThumbnailsList.setAttribute("id", "NextThumbnails");
            nextThumbnailsList.setAttribute("class", "carouselThumbnails");

            var nextCoords = getNextThumbnailCoords();
            nextThumbnailsList.setAttribute("style", "position: absolute; width: " + carouselWidth + "; height: " + carouselHeight + "; list-style-type: none; top: " + nextCoords[0] + "; left: " + nextCoords[1] + ";");

            $(slideshowId + " #Carousel").append(thumbnailsList);
            $(slideshowId + " #Carousel").append(nextThumbnailsList);

            $.slideshow.setCarouselSlideDirection();

            if (carouselButtonPrevious != null)
                $(slideshowId).append(carouselButtonPrevious);

            if (carouselButtonNext != null)
                $(slideshowId).append(carouselButtonNext);
        }
    }

    // Returns a LI HTML carousel element for the specified index.
    function buildCarouselListItem(index) {
        var thumbnailsListItem = document.createElement("li");
        thumbnailsListItem.setAttribute("id", "Thumbnail_" + index);
        var thumbnailsListLink = document.createElement("a");
        thumbnailsListLink.setAttribute("href", "JavaScript:jQuery.slideshow.goToSlide(" + index + ");");
        var thumbnailImage = document.createElement("img");
        var thumbnailImageSrc = slides[index];
        if (index == currentSlideIndex) {
            thumbnailImageSrc = activeThumbnails[index] == null ? thumbnails[index] == null ? slides[index].src : thumbnails[index] : activeThumbnails[index];
            thumbnailImage.setAttribute("class", "activeThumbnail");
        }
        else {
            thumbnailImageSrc = thumbnails[index] == null ? slides[index].src : thumbnails[index];
            thumbnailImage.setAttribute("class", "thumbnail");
        }
        thumbnailImage.setAttribute("src", thumbnailImageSrc);

        thumbnailsListLink.appendChild(thumbnailImage);
        thumbnailsListItem.appendChild(thumbnailsListLink);

        return thumbnailsListItem;
    }

    // Gets the NextThumbnail coods given the current carousel scroll direction
    function getNextThumbnailCoords() {
        switch (carouselCurrentScrollDirection) {
            case "Down":
                return ["-" + carouselHeight, "0"];
            case "Up":
                return [carouselHeight, "0"];
            case "Left":
                return ["0", carouselWidth];
            default:
                return ["0", "-" + carouselWidth];
        }
    }

    // Updates slideshow markup with slide data.
    function updateMarkup(slide) {
        if (slide.alt == null)
            $(slideshowId + " #FrontImage").removeAttr("alt");
        else
            $(slideshowId + " #FrontImage").attr("alt", slide.alt);

        if (slide.href == null)
            $(slideshowId + " #SlideImage").removeAttr("href");
        else
            $(slideshowId + " #SlideImage").attr("href", slide.href);

        if (slide.target == null)
            $(slideshowId + " #SlideImage").removeAttr("target");
        else
            $(slideshowId + " #SlideImage").attr("target", slide.target);

        if (slide.message == null || slide.message.length == 0)
            $(slideshowId + " #Message").css("display", "none");
        else {
            $(slideshowId + " #Message").css("display", "block");
            $(slideshowId + " #Message").html(slide.message);
        }
        $(slideshowId + " #ImageMap").empty();
        if (slide.areas != null && slide.areas.length > 0)
            updateImageMap(slide.areas);

        if ($(slideshowId + " #Dots") != null && $(slideshowId + " #Dots").length > 0)
            updateDots();

        if ($(slideshowId + " #Carousel") != null && $(slideshowId + " #Carousel").length > 0)
            updateCarousel();
    }

    // Updates the slideshow image map with the areas specified.
    function updateImageMap(areas) {
        $.each(areas, function(index, value) {
            var area = document.createElement("area");
            area.setAttribute("alt", value.alt);
            area.setAttribute("href", value.href);
            area.setAttribute("target", value.target);
            area.setAttribute("shape", value.shape);
            area.setAttribute("coords", value.coords);
            $(slideshowId + " #ImageMap").append(area);
        });
    }

    // Updates the slideshow dots markup.
    function updateDots() {
        $(slideshowId + " #Dots img").attr("src", dotButtonInactive.getAttribute("src"));
        $(slideshowId + " #Dot_" + currentSlideIndex + " img").attr("src", dotButtonActive.getAttribute("src"));
        $(slideshowId + " #Dots img").attr("class", dotButtonInactive.getAttribute("class"));
        $(slideshowId + " #Dot_" + currentSlideIndex + " img").attr("class", dotButtonActive.getAttribute("class"));
    }

    // Updates the slideshow carousel markup.
    function updateCarousel() {
        $(slideshowId + " #Thumbnails li").each(function() {
            var thumbnailIndex = parseInt($(this).attr("id").split("_")[1]);
            var thumbnailImageSrc = thumbnails[thumbnailIndex] == null ? slides[thumbnailIndex].src : thumbnails[thumbnailIndex];
            $("img", this).attr("src", thumbnailImageSrc);
            $("img", this).attr("class", "thumbnail");
        });
        var activeThumbnailImageSrc = activeThumbnails[currentSlideIndex] == null ? thumbnails[currentSlideIndex] == null ? slides[currentSlideIndex].src : thumbnails[currentSlideIndex] : activeThumbnails[currentSlideIndex];
        $(slideshowId + " #Thumbnail_" + currentSlideIndex + " img").attr("src", activeThumbnailImageSrc);
        $(slideshowId + " #Thumbnail_" + currentSlideIndex + " img").attr("class", "activeThumbnail");
    }

    // Loads the next group of thumbnails.
    function loadNextThumbnails() {
        $(slideshowId + " #Carousel #NextThumbnails").empty();
        var i;
        if (carouselCursorStart < carouselCursorEnd) {
            for (i = carouselCursorStart; i <= carouselCursorEnd; i++)
                $(slideshowId + " #Carousel #NextThumbnails").append(buildCarouselListItem(i));
        }
        else {
            for (i = carouselCursorStart; i < slides.length; i++)
                $(slideshowId + " #Carousel #NextThumbnails").append(buildCarouselListItem(i));

            for (i = 0; i <= carouselCursorEnd; i++)
                $(slideshowId + " #Carousel #NextThumbnails").append(buildCarouselListItem(i));
        }
    }

    // Swaps out the slides, used when transitionMode is "None"
    function swapSlides(slide) {
        $(slideshowId + " #FrontImage").attr("src", slide.src);
        $(slideshowId + " #BackImage").attr("src", slide.src);
        $(slideshowId + " #BackImage").show();
        updateMarkup(slide);
        startTimer();
    }

    // Cross fades the slides, used when the transitionMode is "Fade"
    function crossFadeSlides(slide) {
        $(slideshowId + " #BackImage").attr("src", slide.src);
        $(slideshowId + " #BackImage").show();
        $(slideshowId + " #FrontImage").fadeOut($.slideshow.settings.transitionDuration, function() {
            $(slideshowId + " #FrontImage").attr("src", slide.src);
            $(slideshowId + " #FrontImage").show();
            updateMarkup(slide);
            startTimer();
        });
    }

    // Slides the slide in the specified direction, used when transitionMode is "Slide"
    function slideSlides(slide) {
        $(slideshowId + " #BackImage").attr("src", slide.src);
        $(slideshowId + " #BackImage").show();
        var direction = $.slideshow.settings.transitionDirection == "Random" ? getRandomDirection() : $.slideshow.settings.transitionDirection;
        $(slideshowId + " #FrontImage").animate(
	        slideDirectionCss($.slideshow.settings.slideshowHeight, $.slideshow.settings.slideshowWidth, direction),
	        $.slideshow.settings.transitionDuration,
	        "swing",
	        function() {
	            $(slideshowId + " #FrontImage").hide();
	            $(slideshowId + " #FrontImage").attr("src", slide.src);
	            $(slideshowId + " #FrontImage").attr("style", "position: absolute; top: 0; left: 0; z-index: 100; border: 0; width: " + $.slideshow.settings.slideshowWidth + "; height: " + $.slideshow.settings.slideshowHeight + ";");
	            $(slideshowId + " #FrontImage").show();
	            updateMarkup(slide);
	            startTimer();
	        });
    }

    // Gets the direction to slide a slideshow element.
    function slideDirectionCss(top, left, direction) {
        switch (direction) {
            case "Up":
                return { "top": "-" + top };
            case "Down":
                return { "top": top };
            case "Right":
                return { "left": left };
            default:
                return { "left": "-" + left };
        }
    }

    // Returns a random direction
    function getRandomDirection() {
        var randomNumber = Math.floor(Math.random() * 4);
        if (randomNumber == 1)
            return "Up";
        else if (randomNumber == 2)
            return "Down";
        else if (randomNumber == 3)
            return "Left";
        else
            return "Right";
    }

    // Sets the carousel direction.
    function setCarouselDirection() {
        if ($.slideshow.settings.carouselScrollDirection == "Infer") {
            if (carouselCurrentScrollDirection == "Down")
                $.slideshow.setCarouselSlideDirection("Up");
            else if (carouselCurrentScrollDirection == "Left")
                $.slideshow.setCarouselSlideDirection("Right");
            else if (carouselCurrentScrollDirection == "Up")
                $.slideshow.setCarouselSlideDirection("Down");
            else if (carouselCurrentScrollDirection == "Right")
                $.slideshow.setCarouselSlideDirection("Left");
        }
        var nextCoords = getNextThumbnailCoords();
        $(slideshowId + " #Carousel #NextThumbnails").css("top", nextCoords[0]);
        $(slideshowId + " #Carousel #NextThumbnails").css("left", nextCoords[1]);
    }

})(jQuery);
